/* rotary.c Reads out the rotary switch
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <pigpiod_if2.h>
#include <unistd.h>
#include <stdbool.h>
#include "rotary.h"

bool rotaryDebug = false;

void rotarySetDebug(bool onOff) {
  rotaryDebug=onOff;
}

int main(int argc, char **argv) {
  static  int retCode;
  static  int pi;
  int dt,sw,clk,clkOld,swOld,swCounter=0,rotCounter=0;

  rotarySetDebug(true);
  
  if ((retCode == pigpio_start(NULL,NULL)) < 0) {
    printf("rotary: Cannot initialize pigpio\n");
    return -1;
  }
  else
    pi=retCode;  
  if (rotaryDebug)
    printf("rotary: pigpio successfully initialized\n");
  /*
    set all pins to input */
  if ((retCode = set_mode(pi,ROTARY_SW,PI_INPUT)) < 0) {
    printf("rotary: Could not set gpio %d to input\n",ROTARY_SW);
    return -1;
  }
  if ((retCode = set_mode(pi,ROTARY_DT,PI_INPUT)) < 0) {
    printf("rotary: Could not set gpio %d to input\n",ROTARY_DT);
    return -1;
  }
  if ((retCode = set_mode(pi,ROTARY_CLK,PI_INPUT)) < 0) {
    printf("rotary: Could not set gpio %d to input\n",ROTARY_CLK);
    return -1;
  }
  /*
    on the push button: switch pull_up on
  */
  if ((retCode = set_pull_up_down(pi,ROTARY_SW,PI_PUD_UP)) < 0) {
    printf("rotary: Could not set pull up on pin  %d \n",ROTARY_SW);
    return -1;
  }

  printf(" clk   dt   sw  rotary counter pushbutton counter\n");
  clkOld=gpio_read(pi,ROTARY_CLK);
  swOld=gpio_read(pi,ROTARY_SW);

  for (;;) {
    sw = gpio_read(pi,ROTARY_SW);
    dt = gpio_read(pi,ROTARY_DT);
    clk = gpio_read(pi,ROTARY_CLK);
    if (clkOld != clk) {	
	clkOld = clk;
      if (dt != clk)
	rotCounter ++;
      else
	rotCounter--;
	printf(" %x     %x     %x          %d         %d\n  ",
	       clk,dt,sw,rotCounter,swCounter);
    };
    if (swOld != sw) {	
	swOld = sw;
	if (sw == 0) {
	  swCounter ++;
	  printf(" %x     %x     %x          %d         %d\n  ",
		 clk,dt,sw,rotCounter,swCounter);

	}
    }
    time_sleep(0.01);
  }
  pigpio_stop(pi);
  return 0;
}
